@@ -1,6 +1,6 @@
 // Vector implementation -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -76,12 +76,13 @@ namespace _GLIBCXX_STD
     struct _Vector_base
     {
       struct _Vector_impl 
-	: public _Alloc {
+      : public _Alloc
+      {
 	_Tp*           _M_start;
 	_Tp*           _M_finish;
 	_Tp*           _M_end_of_storage;
-	_Vector_impl (_Alloc const& __a)
-	  : _Alloc(__a), _M_start(0), _M_finish(0), _M_end_of_storage(0)
+	_Vector_impl(_Alloc const& __a)
+	: _Alloc(__a), _M_start(0), _M_finish(0), _M_end_of_storage(0)
 	{ }
       };
       
@@ -89,13 +90,15 @@ namespace _GLIBCXX_STD
       typedef _Alloc allocator_type;
 
       allocator_type
-      get_allocator() const { return *static_cast<const _Alloc*>(&this->_M_impl); }
+      get_allocator() const
+      { return *static_cast<const _Alloc*>(&this->_M_impl); }
 
-      _Vector_base(const allocator_type& __a) : _M_impl(__a)
+      _Vector_base(const allocator_type& __a)
+      : _M_impl(__a)
       { }
 
       _Vector_base(size_t __n, const allocator_type& __a)
-	: _M_impl(__a)
+      : _M_impl(__a)
       {
 	this->_M_impl._M_start = this->_M_allocate(__n);
 	this->_M_impl._M_finish = this->_M_impl._M_start;
@@ -103,18 +106,22 @@ namespace _GLIBCXX_STD
       }
 
       ~_Vector_base()
-      { _M_deallocate(this->_M_impl._M_start,
-		      this->_M_impl._M_end_of_storage - this->_M_impl._M_start); }
+      { _M_deallocate(this->_M_impl._M_start, this->_M_impl._M_end_of_storage
+		      - this->_M_impl._M_start); }
 
     public:
       _Vector_impl _M_impl;
 
       _Tp*
-      _M_allocate(size_t __n) { return _M_impl.allocate(__n); }
+      _M_allocate(size_t __n)
+      { return _M_impl.allocate(__n); }
 
       void
       _M_deallocate(_Tp* __p, size_t __n)
-      { if (__p) _M_impl.deallocate(__p, __n); }
+      {
+	if (__p)
+	  _M_impl.deallocate(__p, __n);
+      }
     };
 
 
@@ -137,7 +144,7 @@ namespace _GLIBCXX_STD
    *  memory and size allocation.  Subscripting ( @c [] ) access is
    *  also provided as with C-style arrays.
   */
-  template<typename _Tp, typename _Alloc = allocator<_Tp> >
+  template<typename _Tp, typename _Alloc = std::allocator<_Tp> >
     class vector : protected _Vector_base<_Tp, _Alloc>
     {
       // Concept requirements.
@@ -147,19 +154,19 @@ namespace _GLIBCXX_STD
       typedef vector<_Tp, _Alloc>			vector_type;
 
     public:
-      typedef _Tp					value_type;
-      typedef value_type*				pointer;
-      typedef const value_type*				const_pointer;
+      typedef _Tp					 value_type;
+      typedef typename _Alloc::pointer                   pointer;
+      typedef typename _Alloc::const_pointer             const_pointer;
+      typedef typename _Alloc::reference                 reference;
+      typedef typename _Alloc::const_reference           const_reference;
       typedef __gnu_cxx::__normal_iterator<pointer, vector_type> iterator;
       typedef __gnu_cxx::__normal_iterator<const_pointer, vector_type>
       const_iterator;
-      typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
-      typedef std::reverse_iterator<iterator>		reverse_iterator;
-      typedef value_type&				reference;
-      typedef const value_type&				const_reference;
-      typedef size_t					size_type;
-      typedef ptrdiff_t					difference_type;
-      typedef typename _Base::allocator_type		allocator_type;
+      typedef std::reverse_iterator<const_iterator>  const_reverse_iterator;
+      typedef std::reverse_iterator<iterator>		 reverse_iterator;
+      typedef size_t					 size_type;
+      typedef ptrdiff_t					 difference_type;
+      typedef typename _Base::allocator_type		 allocator_type;
 
     protected:
       /** @if maint
@@ -179,7 +186,8 @@ namespace _GLIBCXX_STD
        */
       explicit
       vector(const allocator_type& __a = allocator_type())
-      : _Base(__a) { }
+      : _Base(__a)
+      { }
 
       /**
        *  @brief  Create a %vector with copies of an exemplar element.
@@ -191,8 +199,11 @@ namespace _GLIBCXX_STD
       vector(size_type __n, const value_type& __value,
 	     const allocator_type& __a = allocator_type())
       : _Base(__n, __a)
-      { this->_M_impl._M_finish = std::uninitialized_fill_n(this->_M_impl._M_start,
-						    __n, __value); }
+      {
+	std::__uninitialized_fill_n_a(this->_M_impl._M_start, __n, __value,
+				      this->get_allocator());
+	this->_M_impl._M_finish = this->_M_impl._M_start + __n;
+      }
 
       /**
        *  @brief  Create a %vector with default elements.
@@ -204,8 +215,11 @@ namespace _GLIBCXX_STD
       explicit
       vector(size_type __n)
       : _Base(__n, allocator_type())
-      { this->_M_impl._M_finish = std::uninitialized_fill_n(this->_M_impl._M_start,
-						    __n, value_type()); }
+      {
+	std::__uninitialized_fill_n_a(this->_M_impl._M_start, __n, value_type(),
+				      this->get_allocator());
+	this->_M_impl._M_finish = this->_M_impl._M_start + __n;	
+      }
 
       /**
        *  @brief  %Vector copy constructor.
@@ -218,8 +232,10 @@ namespace _GLIBCXX_STD
        */
       vector(const vector& __x)
       : _Base(__x.size(), __x.get_allocator())
-      { this->_M_impl._M_finish = std::uninitialized_copy(__x.begin(), __x.end(),
-						  this->_M_impl._M_start);
+      { this->_M_impl._M_finish =
+	  std::__uninitialized_copy_a(__x.begin(), __x.end(),
+				      this->_M_impl._M_start,
+				      this->get_allocator());
       }
 
       /**
@@ -243,7 +259,7 @@ namespace _GLIBCXX_STD
 	: _Base(__a)
         {
 	  // Check whether it's an integral type.  If so, it's not an iterator.
-	  typedef typename _Is_integer<_InputIterator>::_Integral _Integral;
+	  typedef typename std::__is_integer<_InputIterator>::__type _Integral;
 	  _M_initialize_dispatch(__first, __last, _Integral());
 	}
 
@@ -253,7 +269,10 @@ namespace _GLIBCXX_STD
        *  not touched in any way.  Managing the pointer is the user's
        *  responsibilty.
        */
-      ~vector() { std::_Destroy(this->_M_impl._M_start, this->_M_impl._M_finish); }
+      ~vector()
+      { std::_Destroy(this->_M_impl._M_start, this->_M_impl._M_finish,
+		      this->get_allocator());
+      }
 
       /**
        *  @brief  %Vector assignment operator.
@@ -297,7 +316,7 @@ namespace _GLIBCXX_STD
         assign(_InputIterator __first, _InputIterator __last)
         {
 	  // Check whether it's an integral type.  If so, it's not an iterator.
-	  typedef typename _Is_integer<_InputIterator>::_Integral _Integral;
+	  typedef typename std::__is_integer<_InputIterator>::__type _Integral;
 	  _M_assign_dispatch(__first, __last, _Integral());
 	}
 
@@ -311,7 +330,8 @@ namespace _GLIBCXX_STD
        *  element order.
        */
       iterator
-      begin() { return iterator (this->_M_impl._M_start); }
+      begin()
+      { return iterator (this->_M_impl._M_start); }
 
       /**
        *  Returns a read-only (constant) iterator that points to the
@@ -319,7 +339,8 @@ namespace _GLIBCXX_STD
        *  element order.
        */
       const_iterator
-      begin() const { return const_iterator (this->_M_impl._M_start); }
+      begin() const
+      { return const_iterator (this->_M_impl._M_start); }
 
       /**
        *  Returns a read/write iterator that points one past the last
@@ -327,7 +348,8 @@ namespace _GLIBCXX_STD
        *  element order.
        */
       iterator
-      end() { return iterator (this->_M_impl._M_finish); }
+      end()
+      { return iterator (this->_M_impl._M_finish); }
 
       /**
        *  Returns a read-only (constant) iterator that points one past
@@ -335,7 +357,8 @@ namespace _GLIBCXX_STD
        *  ordinary element order.
        */
       const_iterator
-      end() const { return const_iterator (this->_M_impl._M_finish); }
+      end() const
+      { return const_iterator (this->_M_impl._M_finish); }
 
       /**
        *  Returns a read/write reverse iterator that points to the
@@ -343,7 +366,8 @@ namespace _GLIBCXX_STD
        *  element order.
        */
       reverse_iterator
-      rbegin() { return reverse_iterator(end()); }
+      rbegin()
+      { return reverse_iterator(end()); }
 
       /**
        *  Returns a read-only (constant) reverse iterator that points
@@ -351,7 +375,8 @@ namespace _GLIBCXX_STD
        *  reverse element order.
        */
       const_reverse_iterator
-      rbegin() const { return const_reverse_iterator(end()); }
+      rbegin() const
+      { return const_reverse_iterator(end()); }
 
       /**
        *  Returns a read/write reverse iterator that points to one
@@ -359,7 +384,8 @@ namespace _GLIBCXX_STD
        *  in reverse element order.
        */
       reverse_iterator
-      rend() { return reverse_iterator(begin()); }
+      rend()
+      { return reverse_iterator(begin()); }
 
       /**
        *  Returns a read-only (constant) reverse iterator that points
@@ -367,16 +393,19 @@ namespace _GLIBCXX_STD
        *  is done in reverse element order.
        */
       const_reverse_iterator
-      rend() const { return const_reverse_iterator(begin()); }
+      rend() const
+      { return const_reverse_iterator(begin()); }
 
       // [23.2.4.2] capacity
       /**  Returns the number of elements in the %vector.  */
       size_type
-      size() const { return size_type(end() - begin()); }
+      size() const
+      { return size_type(end() - begin()); }
 
       /**  Returns the size() of the largest possible %vector.  */
       size_type
-      max_size() const { return size_type(-1) / sizeof(value_type); }
+      max_size() const
+      { return size_type(-1) / sizeof(value_type); }
 
       /**
        *  @brief  Resizes the %vector to the specified number of elements.
@@ -409,7 +438,8 @@ namespace _GLIBCXX_STD
        *  default-constructed.
        */
       void
-      resize(size_type __new_size) { resize(__new_size, value_type()); }
+      resize(size_type __new_size)
+      { resize(__new_size, value_type()); }
 
       /**
        *  Returns the total number of elements that the %vector can
@@ -417,14 +447,16 @@ namespace _GLIBCXX_STD
        */
       size_type
       capacity() const
-      { return size_type(const_iterator(this->_M_impl._M_end_of_storage) - begin()); }
+      { return size_type(const_iterator(this->_M_impl._M_end_of_storage)
+			 - begin()); }
 
       /**
        *  Returns true if the %vector is empty.  (Thus begin() would
        *  equal end().)
        */
       bool
-      empty() const { return begin() == end(); }
+      empty() const
+      { return begin() == end(); }
 
       /**
        *  @brief  Attempt to preallocate enough memory for specified number of
@@ -459,7 +491,8 @@ namespace _GLIBCXX_STD
        *  see at().)
        */
       reference
-      operator[](size_type __n) { return *(begin() + __n); }
+      operator[](size_type __n)
+      { return *(begin() + __n); }
 
       /**
        *  @brief  Subscript access to the data contained in the %vector.
@@ -473,7 +506,8 @@ namespace _GLIBCXX_STD
        *  see at().)
        */
       const_reference
-      operator[](size_type __n) const { return *(begin() + __n); }
+      operator[](size_type __n) const
+      { return *(begin() + __n); }
 
     protected:
       /// @if maint Safety check used only from at().  @endif
@@ -497,7 +531,11 @@ namespace _GLIBCXX_STD
        *  function throws out_of_range if the check fails.
        */
       reference
-      at(size_type __n) { _M_range_check(__n); return (*this)[__n]; }
+      at(size_type __n)
+      {
+	_M_range_check(__n);
+	return (*this)[__n]; 
+      }
 
       /**
        *  @brief  Provides access to the data contained in the %vector.
@@ -511,35 +549,43 @@ namespace _GLIBCXX_STD
        *  function throws out_of_range if the check fails.
        */
       const_reference
-      at(size_type __n) const { _M_range_check(__n); return (*this)[__n]; }
+      at(size_type __n) const
+      {
+	_M_range_check(__n);
+	return (*this)[__n];
+      }
 
       /**
        *  Returns a read/write reference to the data at the first
        *  element of the %vector.
        */
       reference
-      front() { return *begin(); }
+      front()
+      { return *begin(); }
 
       /**
        *  Returns a read-only (constant) reference to the data at the first
        *  element of the %vector.
        */
       const_reference
-      front() const { return *begin(); }
+      front() const
+      { return *begin(); }
 
       /**
        *  Returns a read/write reference to the data at the last
        *  element of the %vector.
        */
       reference
-      back() { return *(end() - 1); }
-
+      back()
+      { return *(end() - 1); }
+      
       /**
        *  Returns a read-only (constant) reference to the data at the
        *  last element of the %vector.
        */
       const_reference
-      back() const { return *(end() - 1); }
+      back() const
+      { return *(end() - 1); }
 
       // [23.2.4.3] modifiers
       /**
@@ -557,7 +603,7 @@ namespace _GLIBCXX_STD
       {
 	if (this->_M_impl._M_finish != this->_M_impl._M_end_of_storage)
 	  {
-	    std::_Construct(this->_M_impl._M_finish, __x);
+	    this->_M_impl.construct(this->_M_impl._M_finish, __x);
 	    ++this->_M_impl._M_finish;
 	  }
 	else
@@ -577,7 +623,7 @@ namespace _GLIBCXX_STD
       pop_back()
       {
 	--this->_M_impl._M_finish;
-	std::_Destroy(this->_M_impl._M_finish);
+	this->_M_impl.destroy(this->_M_impl._M_finish);
       }
 
       /**
@@ -631,7 +677,7 @@ namespace _GLIBCXX_STD
 	       _InputIterator __last)
         {
 	  // Check whether it's an integral type.  If so, it's not an iterator.
-	  typedef typename _Is_integer<_InputIterator>::_Integral _Integral;
+	  typedef typename std::__is_integer<_InputIterator>::__type _Integral;
 	  _M_insert_dispatch(__position, __first, __last, _Integral());
 	}
 
@@ -688,7 +734,8 @@ namespace _GLIBCXX_STD
       {
 	std::swap(this->_M_impl._M_start, __x._M_impl._M_start);
 	std::swap(this->_M_impl._M_finish, __x._M_impl._M_finish);
-	std::swap(this->_M_impl._M_end_of_storage, __x._M_impl._M_end_of_storage);
+	std::swap(this->_M_impl._M_end_of_storage,
+		  __x._M_impl._M_end_of_storage);
       }
 
       /**
@@ -698,7 +745,8 @@ namespace _GLIBCXX_STD
        *  the user's responsibilty.
        */
       void
-      clear() { erase(begin(), end()); }
+      clear()
+      { erase(begin(), end()); }
 
     protected:
       /**
@@ -715,7 +763,8 @@ namespace _GLIBCXX_STD
 	  pointer __result = this->_M_allocate(__n);
 	  try
 	    {
-	      std::uninitialized_copy(__first, __last, __result);
+	      std::__uninitialized_copy_a(__first, __last, __result,
+					  this->get_allocator());
 	      return __result;
 	    }
 	  catch(...)
@@ -735,8 +784,9 @@ namespace _GLIBCXX_STD
         {
 	  this->_M_impl._M_start = _M_allocate(__n);
 	  this->_M_impl._M_end_of_storage = this->_M_impl._M_start + __n;
-	  this->_M_impl._M_finish = std::uninitialized_fill_n(this->_M_impl._M_start,
-						      __n, __value);
+	  std::__uninitialized_fill_n_a(this->_M_impl._M_start, __n, __value,
+					this->get_allocator());
+	  this->_M_impl._M_finish = this->_M_impl._M_end_of_storage;
 	}
 
       // Called by the range constructor to implement [23.1.1]/9
@@ -745,8 +795,8 @@ namespace _GLIBCXX_STD
         _M_initialize_dispatch(_InputIterator __first, _InputIterator __last,
 			       __false_type)
         {
-	  typedef typename iterator_traits<_InputIterator>::iterator_category
-	    _IterCategory;
+	  typedef typename std::iterator_traits<_InputIterator>::
+	    iterator_category _IterCategory;
 	  _M_range_initialize(__first, __last, _IterCategory());
 	}
 
@@ -754,9 +804,9 @@ namespace _GLIBCXX_STD
       template<typename _InputIterator>
         void
         _M_range_initialize(_InputIterator __first,
-			    _InputIterator __last, input_iterator_tag)
+			    _InputIterator __last, std::input_iterator_tag)
         {
-	  for ( ; __first != __last; ++__first)
+	  for (; __first != __last; ++__first)
 	    push_back(*__first);
 	}
 
@@ -764,13 +814,15 @@ namespace _GLIBCXX_STD
       template<typename _ForwardIterator>
         void
         _M_range_initialize(_ForwardIterator __first,
-			    _ForwardIterator __last, forward_iterator_tag)
+			    _ForwardIterator __last, std::forward_iterator_tag)
         {
-	  size_type __n = std::distance(__first, __last);
+	  const size_type __n = std::distance(__first, __last);
 	  this->_M_impl._M_start = this->_M_allocate(__n);
 	  this->_M_impl._M_end_of_storage = this->_M_impl._M_start + __n;
-	  this->_M_impl._M_finish = std::uninitialized_copy(__first, __last,
-						    this->_M_impl._M_start);
+	  this->_M_impl._M_finish =
+	    std::__uninitialized_copy_a(__first, __last,
+					this->_M_impl._M_start,
+					this->get_allocator());
 	}
 
 
@@ -792,8 +844,8 @@ namespace _GLIBCXX_STD
         _M_assign_dispatch(_InputIterator __first, _InputIterator __last,
 			   __false_type)
         {
-	  typedef typename iterator_traits<_InputIterator>::iterator_category
-	    _IterCategory;
+	  typedef typename std::iterator_traits<_InputIterator>::
+	    iterator_category _IterCategory;
 	  _M_assign_aux(__first, __last, _IterCategory());
 	}
 
@@ -801,13 +853,13 @@ namespace _GLIBCXX_STD
       template<typename _InputIterator>
         void
         _M_assign_aux(_InputIterator __first, _InputIterator __last,
-		      input_iterator_tag);
+		      std::input_iterator_tag);
 
       // Called by the second assign_dispatch above
       template<typename _ForwardIterator>
         void
         _M_assign_aux(_ForwardIterator __first, _ForwardIterator __last,
-		      forward_iterator_tag);
+		      std::forward_iterator_tag);
 
       // Called by assign(n,t), and the range assign when it turns out
       // to be the same thing.
@@ -833,8 +885,8 @@ namespace _GLIBCXX_STD
         _M_insert_dispatch(iterator __pos, _InputIterator __first,
 			   _InputIterator __last, __false_type)
         {
-	  typedef typename iterator_traits<_InputIterator>::iterator_category
-	    _IterCategory;
+	  typedef typename std::iterator_traits<_InputIterator>::
+	    iterator_category _IterCategory;
 	  _M_range_insert(__pos, __first, __last, _IterCategory());
 	}
 
@@ -842,13 +894,13 @@ namespace _GLIBCXX_STD
       template<typename _InputIterator>
         void
         _M_range_insert(iterator __pos, _InputIterator __first,
-			_InputIterator __last, input_iterator_tag);
+			_InputIterator __last, std::input_iterator_tag);
 
       // Called by the second insert_dispatch above
       template<typename _ForwardIterator>
         void
         _M_range_insert(iterator __pos, _ForwardIterator __first,
-			_ForwardIterator __last, forward_iterator_tag);
+			_ForwardIterator __last, std::forward_iterator_tag);
 
       // Called by insert(p,n,x), and the range insert when it turns out to be
       // the same thing.
@@ -873,11 +925,9 @@ namespace _GLIBCXX_STD
   */
   template<typename _Tp, typename _Alloc>
     inline bool
-    operator==(const vector<_Tp,_Alloc>& __x, const vector<_Tp,_Alloc>& __y)
-    {
-      return __x.size() == __y.size() &&
-             std::equal(__x.begin(), __x.end(), __y.begin());
-    }
+    operator==(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
+    { return (__x.size() == __y.size()
+	      && std::equal(__x.begin(), __x.end(), __y.begin())); }
 
   /**
    *  @brief  Vector ordering relation.
@@ -892,40 +942,38 @@ namespace _GLIBCXX_STD
   */
   template<typename _Tp, typename _Alloc>
     inline bool
-    operator<(const vector<_Tp,_Alloc>& __x, const vector<_Tp,_Alloc>& __y)
-    {
-      return std::lexicographical_compare(__x.begin(), __x.end(),
-					  __y.begin(), __y.end());
-    }
+    operator<(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
+    { return std::lexicographical_compare(__x.begin(), __x.end(),
+					  __y.begin(), __y.end()); }
 
   /// Based on operator==
   template<typename _Tp, typename _Alloc>
     inline bool
-    operator!=(const vector<_Tp,_Alloc>& __x, const vector<_Tp,_Alloc>& __y)
+    operator!=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
     { return !(__x == __y); }
 
   /// Based on operator<
   template<typename _Tp, typename _Alloc>
     inline bool
-    operator>(const vector<_Tp,_Alloc>& __x, const vector<_Tp,_Alloc>& __y)
+    operator>(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
     { return __y < __x; }
 
   /// Based on operator<
   template<typename _Tp, typename _Alloc>
     inline bool
-    operator<=(const vector<_Tp,_Alloc>& __x, const vector<_Tp,_Alloc>& __y)
+    operator<=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
     { return !(__y < __x); }
 
   /// Based on operator<
   template<typename _Tp, typename _Alloc>
     inline bool
-    operator>=(const vector<_Tp,_Alloc>& __x, const vector<_Tp,_Alloc>& __y)
+    operator>=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
     { return !(__x < __y); }
 
   /// See std::vector::swap().
   template<typename _Tp, typename _Alloc>
     inline void
-    swap(vector<_Tp,_Alloc>& __x, vector<_Tp,_Alloc>& __y)
+    swap(vector<_Tp, _Alloc>& __x, vector<_Tp, _Alloc>& __y)
     { __x.swap(__y); }
 } // namespace std
 
