@@ -38,36 +38,21 @@ void test01()
 	     traits_type::state_type>);
 
   basic_filebuf<pod_uchar> fb;
+  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
-  fb.open(name, ios_base::out | ios_base::trunc);
-  fb.sputc(pod_uchar::from<char>('b'));
-  fb.sputc(pod_uchar::from<char>(0xff));
+  fb.open(name, ios_base::in | ios_base::out | ios_base::trunc);
+  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::cur);
   fb.sputc(pod_uchar::from<char>('a'));
-  fb.sputc(pod_uchar::from<char>(0xfc));
+  fb.sputc(pod_uchar::from<char>(0xff));
   fb.sputc(pod_uchar::from<char>(0));
   fb.sputc(pod_uchar::from<char>(0));
 
-  fb.close();
-  fb.open(name, ios_base::in);
+  // Check that seekpos restores the state correctly
+  fb.pubseekpos(pos);
 
-  fb.sbumpc();
-  fb.sbumpc();
-  traits_type::pos_type pos1 = fb.pubseekoff(0, ios_base::cur);
-  fb.sbumpc();
-  fb.sbumpc();
-
-  // Check that seekpos returns the correct state
-  traits_type::pos_type pos2 = fb.pubseekpos(pos1);
   traits_type::int_type c = fb.sbumpc();
   VERIFY( c != traits_type::eof() );
-  VERIFY( traits_type::eq(traits_type::to_char_type(c),
-			  pod_uchar::from<char>('a')) );
-  fb.sbumpc();
-
-  fb.pubseekpos(pos2);
-  c = fb.sbumpc();
-  VERIFY( c != traits_type::eof() );
   VERIFY( traits_type::eq(traits_type::to_char_type(c),
 			  pod_uchar::from<char>('a')) );
 
