@@ -21,32 +21,29 @@
 #include <fstream>
 #include <testsuite_hooks.h>
 
-const char name_04[] = "tmp_11389-4";
+const char name_01[] = "tmp_11389-1";
 
-void test04()
+void test01()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
 
   filebuf fbout;
-  fbout.open(name_04, ios_base::out);
+  fbout.open(name_01, ios_base::out);
   fbout.sputc('a');
-  fbout.sputc('b');
   fbout.close();
   
   wfilebuf fbin;
   locale loc(__gnu_test::try_named_locale("en_US.UTF-8"));
   fbin.pubimbue(loc);
-  fbin.pubsetbuf(0, 0);
-  fbin.open(name_04, ios_base::in);
+  fbin.open(name_01, ios_base::in);
   VERIFY( fbin.sbumpc() == L'a' );
-  VERIFY( fbin.sbumpc() == L'b' );
   VERIFY( fbin.sgetc() == wfilebuf::traits_type::eof() );
   fbin.close();
 }
 
 int main()
 {
-  test04();
+  test01();
   return 0;
 }
