@@ -26,8 +26,9 @@
 int main()
 {
   using namespace __gnu_test;
-  // cplus-dem FAIL
-  verify_demangle("_Z1fA37_iPS_", "f(int [37], int (*) [37])");
+
+  // cplus-dem FAIL  
+  verify_demangle("_Z1jM1AFivEPS1_", "j(int (A::*)(), int (A::**)())");
 
   return 0;
 }
